@@ -40,4 +40,12 @@ int github_get_notifications(struct gcli_ctx *ctx, int max,
                              struct gcli_notification_list *out);
 int github_notification_mark_as_read(struct gcli_ctx *ctx, char const *id);
 
+int github_notification_get_issue(struct gcli_ctx *ctx,
+                                  struct gcli_notification const *notification,
+                                  struct gcli_issue *out);
+
+int github_notification_get_comments(struct gcli_ctx *const ctx,
+                                     struct gcli_notification const *const notification,
+                                     struct gcli_comment_list *const out);
+
 #endif /* GITHUB_STATUS_H */
