@@ -39,5 +39,12 @@
 int gitlab_get_notifications(struct gcli_ctx *ctx, int max,
                              struct gcli_notification_list *out);
 int gitlab_notification_mark_as_read(struct gcli_ctx *ctx, char const *id);
+int gitlab_notification_get_issue(struct gcli_ctx *ctx,
+                                  struct gcli_notification const *const notification,
+                                  struct gcli_issue *out);
+
+int gitlab_notification_get_comments(struct gcli_ctx *ctx,
+                                     struct gcli_notification const *const notifcation,
+                                     struct gcli_comment_list *const out);
 
 #endif /* GITLAB_STATUS_H */
