@@ -34,17 +34,38 @@
 #include <config.h>
 #endif
 
+#include <gcli/comments.h>
 #include <gcli/gcli.h>
+#include <gcli/issues.h>
 
 #include <stdlib.h>
 
+enum gcli_notification_target_type {
+	GCLI_NOTIFICATION_TARGET_INVALID = 0,
+
+	GCLI_NOTIFICATION_TARGET_ISSUE,
+	GCLI_NOTIFICATION_TARGET_PULL_REQUEST,
+	GCLI_NOTIFICATION_TARGET_COMMIT,
+	GCLI_NOTIFICATION_TARGET_EPIC,
+	GCLI_NOTIFICATION_TARGET_REPOSITORY,
+
+	MAX_GCLI_NOTIFICATION_TARGET,
+};
+
 struct gcli_notification {
 	char *id;
 	char *title;
 	char *reason;
 	char *date;
-	char *type;
+	enum gcli_notification_target_type type;
 	char *repository;
+
+	/* target specific data */
+	struct {
+		gcli_id id;   /* The internal ID of the target data */
+		gcli_id project_id;
+		char *url;
+	} target;
 };
 
 struct gcli_notification_list {
@@ -57,5 +78,13 @@ int gcli_get_notifications(struct gcli_ctx *ctx, int count,
 int gcli_notification_mark_as_read(struct gcli_ctx *ctx, char const *id);
 void gcli_free_notification(struct gcli_notification *);
 void gcli_free_notifications(struct gcli_notification_list *);
+char const *gcli_notification_target_type_str(enum gcli_notification_target_type type);
+int gcli_notification_get_issue(struct gcli_ctx *ctx,
+                                struct gcli_notification const *notification,
+                                struct gcli_issue *out);
+
+int gcli_notification_get_comments(struct gcli_ctx *ctx,
+                                   struct gcli_notification const *const notification,
+                                   struct gcli_comment_list *comments);
 
 #endif /* STATUS_H */
