@@ -39,6 +39,7 @@
 #include <gcli/cmd/interactive.h>
 #include <gcli/cmd/pipelines.h>
 #include <gcli/cmd/pulls.h>
+#include <gcli/cmd/pull_reviews.h>
 #include <gcli/cmd/table.h>
 
 #include <gcli/comments.h>
@@ -97,6 +98,8 @@ usage(void)
 	fprintf(stderr, "  patch                  Display changes as patch series\n");
 	fprintf(stderr, "  title <new-title>      Change the title of the pull request\n");
 	fprintf(stderr, "  request-review <user>  Add <user> as a reviewer of the PR\n");
+	if (gcli_config_enable_experimental(g_clictx))
+		fprintf(stderr, "  review                 Start a review of this PR\n");
 
 	fprintf(stderr, "\n");
 	version();
@@ -1072,6 +1075,22 @@ action_title(struct action_ctx *const ctx)
 	ctx->argv += 1;
 }
 
+static void
+action_review(struct action_ctx *ctx)
+{
+	if (gcli_config_enable_experimental(g_clictx) == false) {
+		errx(1, "gcli: error: review is not available because it is "
+		     "considered experimental. To enable this feature set "
+		     "enable-experimental in your gcli config file or "
+		     "set GCLI_ENABLE_EXPERIMENTAL in your environment.");
+	}
+
+	ctx->argc -= 1;
+	ctx->argv += 1;
+
+	do_review_session(ctx->owner, ctx->repo, ctx->pr);
+}
+
 static struct action {
 	char const *name;
 	void (*fn)(struct action_ctx *ctx);
@@ -1092,6 +1111,7 @@ static struct action {
 	{ .name = "milestone",      .fn = action_milestone      },
 	{ .name = "request-review", .fn = action_request_review },
 	{ .name = "title",          .fn = action_title          },
+	{ .name = "review",         .fn = action_review         },
 };
 
 static size_t const actions_size = ARRAY_SIZE(actions);
