@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022, 2023, 2024 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,6 +31,7 @@
 
 #include <gcli/cmd/cmd.h>
 #include <gcli/cmd/status.h>
+#include <gcli/cmd/status_interactive.h>
 
 #include <gcli/status.h>
 
@@ -42,17 +43,19 @@ static void
 usage(void)
 {
 	fprintf(stderr, "usage: gcli status -m id\n");
-	fprintf(stderr, "       gcli status [-n number]\n");
+	fprintf(stderr, "       gcli status -l [-n number]\n");
+	fprintf(stderr, "       gcli status\n");
 	fprintf(stderr, "OPTIONS:\n");
-	fprintf(stderr, "  -n number       Number of messages to fetch\n");
-	fprintf(stderr, "  -m id           Mark the given message as read\n");
+	fprintf(stderr, "  -l          Print list of todo items and exit\n");
+	fprintf(stderr, "  -n number   Number of messages to fetch\n");
+	fprintf(stderr, "  -m id       Mark the given message as read\n");
 	fprintf(stderr, "\n");
 	version();
 	copyright();
 }
 
 int
-gcli_status(int const count)
+gcli_status_list(int const count)
 {
 	struct gcli_notification_list list = {0};
 	int rc = 0;
@@ -71,16 +74,22 @@ void
 gcli_print_notifications(struct gcli_notification_list const *const list)
 {
 	for (size_t i = 0; i < list->notifications_size; ++i) {
-		printf("%s - %s - %s - %s",
+		printf("%s - %s - %s",
 		       list->notifications[i].id,
 		       list->notifications[i].repository,
-		       list->notifications[i].type, list->notifications[i].date);
+		       gcli_notification_target_type_str(list->notifications[i].type));
 
-		if (list->notifications[i].reason) {
-			printf(" - %s\n", list->notifications[i].reason);
-		} else {
-			printf("\n");
-		}
+		/* XXX what about target_id being zero? does any forge generate
+		 *     zero for valid IDs? */
+		if (list->notifications[i].target.id)
+			printf(" %"PRIid, list->notifications[i].target.id);
+
+		printf(" - %s", list->notifications[i].date);
+
+		if (list->notifications[i].reason)
+			printf(" - %s", list->notifications[i].reason);
+
+		printf("\n");
 
 		pretty_print(list->notifications[i].title, 4, 80, stdout);
 		putchar('\n');
@@ -90,10 +99,8 @@ gcli_print_notifications(struct gcli_notification_list const *const list)
 int
 subcommand_status(int argc, char *argv[])
 {
-	int   count  = 30;
-	int   ch     = 0;
+	int count = 30, ch = 0, mark = 0, list = 0;
 	char *endptr = NULL;
-	int   mark   = 0;
 
 	const struct option options[] = {
 		{ .name    = "count",
@@ -104,10 +111,14 @@ subcommand_status(int argc, char *argv[])
 		  .has_arg = no_argument,
 		  .flag    = &mark,
 		  .val     = 1 },
+		{ .name    = "list",
+		  .has_arg = no_argument,
+		  .flag    = &list,
+		  .val     = 1 },
 		{0}
 	};
 
-	while ((ch = getopt_long(argc, argv, "n:m", options, NULL)) != -1) {
+	while ((ch = getopt_long(argc, argv, "n:ml", options, NULL)) != -1) {
 		switch (ch) {
 		case 'n': {
 			count = strtol(optarg, &endptr, 10);
@@ -117,6 +128,9 @@ subcommand_status(int argc, char *argv[])
 		case 'm': {
 			mark = 1;
 		} break;
+		case 'l': {
+			list = 1;
+		} break;
 		default:
 			usage();
 			return EXIT_FAILURE;
@@ -126,9 +140,16 @@ subcommand_status(int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
-	if (!mark) {
-		gcli_status(count);
-	} else {
+	if (list) {
+		if (mark) {
+			fprintf(stderr, "gcli: cannot use -m and -l together\n");
+			usage();
+			return EXIT_FAILURE;
+		}
+
+		gcli_status_list(count);
+
+	} else if (mark) {
 		if (count != 30)
 			warnx("gcli: ignoring -n/--count argument");
 
@@ -147,7 +168,10 @@ subcommand_status(int argc, char *argv[])
 		if (gcli_notification_mark_as_read(g_clictx, argv[0]) < 0)
 			errx(1, "gcli: error: failed to mark the notification as read: %s",
 			     gcli_get_error(g_clictx));
+	} else {
+		return gcli_status_interactive();
 	}
 
 	return EXIT_SUCCESS;
 }
+
