@@ -93,7 +93,7 @@ gitea_get_milestone(struct gcli_ctx *ctx, char const *const owner,
 		json_close(&stream);
 	}
 
-	free(buffer.data);
+	gcli_fetch_buffer_free(&buffer);
 	free(url);
 
 	return rc;
